
#ifndef __PlanetCNCLib_H__
#define __PlanetCNCLib_H__


#include "PlanetCNCLibTypeDef.h"


#ifdef __cplusplus
extern "C" {
#endif

#ifndef PCNC_CALL
#	ifdef _WIN32
#		define PCNC_CALL __cdecl
#	else
#		define PCNC_CALL
#	endif
#endif

#ifndef PCNC_EXPORT
#	ifdef _WIN32
#		define PCNC_EXPORT __declspec(dllexport)
#	else
#		define PCNC_EXPORT __attribute__((visibility("default"))) 
#	endif
#endif


	//Memory
	PCNC_EXPORT void FreeString(char* str);
	PCNC_EXPORT void FreeStringW(wchar_t* strw);

	//Callback
	PCNC_EXPORT void SetInitialiseCB(__InitialiseCB cb);
	PCNC_EXPORT void SetRefreshCB(__RefreshCB cb);
	PCNC_EXPORT void SetOpenCB(__OpenCB cb);
	PCNC_EXPORT void SetOutputCB(__OutputCB cb);
	PCNC_EXPORT void SetOutputWCB(__OutputWCB cb);
	PCNC_EXPORT void SetIdleCB(__IdleCB cb);
	PCNC_EXPORT void SetLineNumCB(__LineNumCB cb);

	//Run & Exit
	PCNC_EXPORT int Run(bool hideUI);
	PCNC_EXPORT int RunProfile(bool hideUI, const char* profile);
	PCNC_EXPORT int RunProfileW(bool hideUI, const wchar_t* profile);
	PCNC_EXPORT bool Exit();
	PCNC_EXPORT void ExitForce();

	//Run status
	PCNC_EXPORT unsigned int GetVer();
	PCNC_EXPORT char* GetLibPath();
	PCNC_EXPORT wchar_t* GetLibPathW();

	PCNC_EXPORT bool IsRunning();
	PCNC_EXPORT bool IsRunningExt();
	PCNC_EXPORT bool IsInitialized();
	PCNC_EXPORT unsigned long long GetDateTime();

	//Version & Profile Info
	PCNC_EXPORT char* GetVersionString();
	PCNC_EXPORT wchar_t* GetVersionStringW();
	PCNC_EXPORT char* GetProfileNameString();
	PCNC_EXPORT wchar_t* GetProfileNameStringW();
	PCNC_EXPORT char* GetProfilePathString();
	PCNC_EXPORT wchar_t* GetProfilePathStringW();
	PCNC_EXPORT char* GetDescriptionString();
	PCNC_EXPORT wchar_t* GetDescriptionStringW();

	//Screen
	PCNC_EXPORT bool IsVisible();
	PCNC_EXPORT bool SetVisible(bool visible);
	PCNC_EXPORT bool Show();
	PCNC_EXPORT bool Hide();

	PCNC_EXPORT bool SetProgress(double dbl);
	PCNC_EXPORT bool SetStatus(const char* str);
	PCNC_EXPORT bool SetStatusW(const wchar_t* strw);

	PCNC_EXPORT int Msg(const char* str);
	PCNC_EXPORT int MsgW(const wchar_t* strw);
	PCNC_EXPORT bool Print(const char* str);
	PCNC_EXPORT bool PrintW(const wchar_t* strw);

	//Parameters & Eval
	PCNC_EXPORT double GetParam(const char* param);
	PCNC_EXPORT double GetParamW(const wchar_t* param);
	PCNC_EXPORT char* GetParamStr(const char* param);
	PCNC_EXPORT wchar_t* GetParamStrW(const wchar_t* param);

	PCNC_EXPORT bool SetParam(const char* param, double value);
	PCNC_EXPORT bool SetParamW(const wchar_t* param, double value);
	PCNC_EXPORT bool SetParamStr(const char* param, const char* value);
	PCNC_EXPORT bool SetParamStrW(const wchar_t* param, const wchar_t* value);

	PCNC_EXPORT double Evaluate(const char* expr);
	PCNC_EXPORT double EvaluateW(const wchar_t* expr);

	//State
	PCNC_EXPORT bool IsLicenseValid();
	PCNC_EXPORT bool IsUIReady();
	PCNC_EXPORT bool IsControllerReady();
	PCNC_EXPORT bool IsControllerRunning();
	PCNC_EXPORT bool IsProgramLoaded();

	PCNC_EXPORT bool IsIdle();
	PCNC_EXPORT bool IsRun();
	PCNC_EXPORT bool IsEStop();
	PCNC_EXPORT bool IsStop();
	PCNC_EXPORT bool IsPause();
	PCNC_EXPORT bool IsProg();
	PCNC_EXPORT bool IsCmd();
	PCNC_EXPORT bool IsJog();

	//Machine Command Status
	PCNC_EXPORT bool IsEStopEnabled();
	PCNC_EXPORT bool IsStopEnabled();
	PCNC_EXPORT bool IsPauseEnabled();
	PCNC_EXPORT bool IsStartEnabled();
	PCNC_EXPORT bool IsOutputEnabled();
	PCNC_EXPORT bool IsCodeEnabled();
	PCNC_EXPORT bool IsCodeExEnabled();
	PCNC_EXPORT bool IsOpenEnabled();
	PCNC_EXPORT bool IsCloseEnabled();

	//Machine Commands
	PCNC_EXPORT bool EStop(bool b);
	PCNC_EXPORT bool EStopToggle();
	PCNC_EXPORT bool Stop();
	PCNC_EXPORT bool Pause(bool b);
	PCNC_EXPORT bool PauseToggle();
	PCNC_EXPORT bool Start();
	PCNC_EXPORT bool Open();
	PCNC_EXPORT bool Close();

	PCNC_EXPORT bool MoveAxis(double speed, int axis, double val);
	PCNC_EXPORT bool Move3(double speed, double x, double y, double z);
	PCNC_EXPORT bool Move9(double speed, double x, double y, double z, double a, double b, double c, double u, double v, double w);

	PCNC_EXPORT bool Output(int num, bool value);
	PCNC_EXPORT bool OutputPWM(int num, double freq, double duty);
	PCNC_EXPORT bool OutputRC(int num, double value);

	PCNC_EXPORT bool ExtOut(int ssel, int num, bool value);

	//Machine Commands Generic
	PCNC_EXPORT int GetCmdCount();
	PCNC_EXPORT int GetCmdId(const char* str);
	PCNC_EXPORT int GetCmdIdW(const wchar_t* strw);

	PCNC_EXPORT bool IsCmdEnabled(int id);
	PCNC_EXPORT bool IsCmdChecked(int id);

	PCNC_EXPORT char* GetCmdName(int id);
	PCNC_EXPORT wchar_t* GetCmdNameW(int id);
	PCNC_EXPORT char* GetCmdDisplayName(int id);
	PCNC_EXPORT wchar_t* GetCmdDisplayNameW(int id);

	PCNC_EXPORT bool CmdExec(int id);

	PCNC_EXPORT bool CmdExecStr(int id, const char* str);
	PCNC_EXPORT bool CmdExecStrW(int id, const wchar_t* strw);
	PCNC_EXPORT bool CmdExecVal(int id, double val);
	PCNC_EXPORT bool CmdExecIntVal(int id, int num, double val);
	PCNC_EXPORT bool CmdExecIntVal2(int id, int num, double val, double val2);

	PCNC_EXPORT bool Cmd(const char* str);
	PCNC_EXPORT bool CmdW(const wchar_t* strw);

	//Helpers
	PCNC_EXPORT bool OpenFn(const char* str);
	PCNC_EXPORT bool OpenFnW(const wchar_t* strw);
	PCNC_EXPORT bool OpenCode(const char* str);
	PCNC_EXPORT bool OpenCodeW(const wchar_t* strw);

	PCNC_EXPORT int TestFn(const char* str);
	PCNC_EXPORT int TestFnW(const wchar_t* strw);
	PCNC_EXPORT int TestCode(const char* str);
	PCNC_EXPORT int TestCodeW(const wchar_t* strw);

	PCNC_EXPORT bool StartFn(const char* str);
	PCNC_EXPORT bool StartFnW(const wchar_t* strw);
	PCNC_EXPORT bool StartCode(const char* str);
	PCNC_EXPORT bool StartCodeW(const wchar_t* strw);

	PCNC_EXPORT double Python(const char* str, bool useThread);
	PCNC_EXPORT double PythonW(const wchar_t* strw, bool useThread);

	//G-Code
	PCNC_EXPORT char* GetFn();
	PCNC_EXPORT wchar_t* GetFnW();
	PCNC_EXPORT	int GetLineCount();
	PCNC_EXPORT	char* GetLine(int row);
	PCNC_EXPORT	wchar_t* GetLineW(int row);
	PCNC_EXPORT	int GetLineNum();

	//Points
	PCNC_EXPORT int PointsCount();
	PCNC_EXPORT bool PointsClear();
	PCNC_EXPORT bool PointsAdd3(double x, double y, double z);
	PCNC_EXPORT bool PointsAdd9(double x, double y, double z, double a, double b, double c, double u, double v, double w);

	//Info
	PCNC_EXPORT	bool InfoIsInitialized();
	PCNC_EXPORT	int InfoSerial();
	PCNC_EXPORT	int InfoHWVersion();
	PCNC_EXPORT	int InfoSWVersion();
	PCNC_EXPORT	bool InfoIsVersionValid();

	PCNC_EXPORT unsigned int InfoBufferAvailable();
	PCNC_EXPORT double InfoBufferUtilization();

	PCNC_EXPORT double InfoWorkPosition(int n);
	PCNC_EXPORT bool InfoWorkPosition3(double* x, double* y, double* z);
	PCNC_EXPORT bool InfoWorkPosition9(double* x, double* y, double* z, double* a, double* b, double* c, double* u, double* v, double* w);

	PCNC_EXPORT double InfoMotorPosition(int n);
	PCNC_EXPORT bool InfoMotorPosition3(double* x, double* y, double* z);
	PCNC_EXPORT bool InfoMotorPosition9(double* x, double* y, double* z, double* a, double* b, double* c, double* u, double* v, double* w);

	PCNC_EXPORT double InfoSpeed();
	PCNC_EXPORT double InfoAcceleration();

	PCNC_EXPORT double InfoSpindle();
	PCNC_EXPORT double InfoSpindleIdx();
	PCNC_EXPORT double InfoSpindleEnc();
	PCNC_EXPORT double InfoSpindleSet();

	PCNC_EXPORT unsigned int InfoOutput();
	PCNC_EXPORT bool InfoOutputPin(int num);
	PCNC_EXPORT double InfoOutputFreq(int num);
	PCNC_EXPORT double InfoOutputDuty(int num);

	PCNC_EXPORT unsigned int InfoInput();
	PCNC_EXPORT bool InfoInputPin(int num);

	PCNC_EXPORT unsigned int InfoLimit();
	PCNC_EXPORT bool InfoLimitPin(int num);

	PCNC_EXPORT unsigned int InfoJog();
	PCNC_EXPORT bool InfoJogPin(int num);
	PCNC_EXPORT unsigned int InfoJogPot();

	PCNC_EXPORT unsigned int InfoAux(int num);

	PCNC_EXPORT bool InfoSpeedOverrideEnabled();
	PCNC_EXPORT bool InfoSpindleOverrideEnabled();

	//Jog
	PCNC_EXPORT bool JogStop();
	PCNC_EXPORT bool Jog(bool step, double x, double y, double z);
	PCNC_EXPORT bool Jog9(bool step, double x, double y, double z, double a, double b, double c, double u, double v, double w);
	PCNC_EXPORT bool JogAxis(bool step, int axis, double dir, double param);

	//THC
	PCNC_EXPORT bool THC(bool arcok, int dir);

	//G-Code Lines
	PCNC_EXPORT void* LineListCreate();
	PCNC_EXPORT void LineListFree(void* ptr);

	PCNC_EXPORT bool LineListClear(void* ptr);
	PCNC_EXPORT int LineListCount(void* ptr);

	PCNC_EXPORT bool LineListAddAllowed(void* ptr);
	PCNC_EXPORT bool LineListAdd(void* ptr, const char* str);
	PCNC_EXPORT bool LineListAddW(void* ptr, const wchar_t* str);

	PCNC_EXPORT char* LineListGet(void* ptr, int i);
	PCNC_EXPORT wchar_t* LineListGetW(void* ptr, int i);

	PCNC_EXPORT bool LineListLoadFromFile(void* ptr, const char* fn);
	PCNC_EXPORT bool LineListLoadFromFileW(void* ptr, const wchar_t* fn);

	PCNC_EXPORT bool OpenLineList(void* ptr);
	PCNC_EXPORT int TestLineList(void* ptr);
	PCNC_EXPORT bool StartLineList(void* ptr);



#ifdef __cplusplus
}
#endif

#endif 

